function ydot = pkModel(t, y, u, age, weight)
% Implements a differential equation for a pharmacokinetic model based on the
% Paedfusor Data Set. 
%
%   ydot = pkpdDE(t, y, u, age, weight)
%
% Implements ydot = A * y + B * u, where A and B are matricies found in 

% Maidens, John N., et al. "Lagrangian methods for approximating the viability
% kernel in high-dimensional systems." Automatica 49.7 (2013): 2017-2029.
% 
% The parameter values used are given in
%
% Absalom, A., and G. Kenny. "'Paedfusor' pharmacokinetic data set." British
% journal of anaesthesia 95.1 (2005): 110-110.  
%
% Input parameters:
%
%   t: Scalar. The current time. Used if input u is a function of time.
%
%   y: Column vector. The current state.
%
%   u: Scalar or function handle.  Input.  If a function handle, then that
%   function is called with t as an input argument.  Optional, default = 0.
%
%   age: Scalar. The age of the patient in years. Optional, default = 11.
%
%   weight: Scalar. The weight of the patient in kg. Optional, default = 35.
%
% Output parameters:
%
%   ydot: Column vector the same size as y.  The state derivative.
%
% Note: if u is a constant, there exists a stable fixed point at 
% ( (u/(V1*k10), u/(V1*k10), u/(V1*k10) )

% -----------------------------------------------------------------------------
%% Default parameters
if (nargin < 3)
    u_val = 0;
end
if (nargin < 4)
    age = 11;
end
if (nargin < 5)
    weight = 35;
end

if isa(u, 'function_handle')
    u_val = u(t);
else
    u_val = u;
end

% -----------------------------------------------------------------------------
%% Paedfusor model parameters 
% These parameters are from the paper cited above
switch logical(true)
    case (age > 0) & (age < 13)
        V1 = 458.4*weight;
        k10 = 0.1527*weight^-0.3;
    case age == 13
        V1 = 400*weight;
        k10 = 0.0678;
    case age == 14
        V1 = 342*weight;
        k10 = 0.0792;
    case age == 15
        V1 = 284*weight;
        k10 = 0.0954;
    case age == 16
        V1 = 228.57*weight;
        k10 = 0.119;
    otherwise
        error('Invalid age. Must be between 1 and 16 (inclusive).');
end % switch

k12=0.114;
k13=0.0419;
k21=0.055;
k31=0.0033;
% Paedfusor paper has additional parameters that are not used in our model


A = [-(k10+k12+k13)  k12  k13;
           k21      -k21   0 ;
           k31        0  -k31 ];
B = [1/V1;
      0  ;
      0  ];

ydot = A * y + B * u_val;
    
end % function 
